###############RAPTT Null Distribution ###############
##Package required
library(mvtnorm)

# Simulation of RAPTT Statisitc based on R1 
RAPTT.Stats.R1=function(X,Y,m)   ##Input DATA matrix X and Y and m=number of projection
{
n1=dim(X)[1]
n2=dim(Y)[1]
p=dim(X)[2]
##Computing proposed projection dimension
k1=1:(n1+n2-2);
k=which(qf(0.95, k1, n1+n2-k1-1)==min(qf(0.95, k1, n1+n2-k1-1)));
#Computing Sufficient Statsitic 
Xbar=colMeans(X)
Ybar=colMeans(Y)
S=(var(X)*(n1-1)+var(Y)*(n2-1))/(n1+n2-2)
theta=matrix(0,m,1)
for (l in 1:m)
{
#Generation of random projection matrix distributed as Haar Measure
Q=qr.Q(qr(matrix(rnorm(p*k),p)))
R=qr.Q(qr(matrix(rnorm(p*k),p)))
L=diag(diag(R)/abs(diag(R)))
R1=Q%*%L
#Randomized Hotelling T2
T2=((Xbar-Ybar)%*%R1%*%(solve(t(R1)%*%S%*%R1))%*%t(R1)%*%(Xbar-Ybar))*(n1*n2/(n1+n2))
#p-value of randomized T2 
theta[l]=1-pf(((n1+n2-2-k+1)/(k*(n1+n2-2)))*T2,k,n1+n2-2-k+1)
}
return(mean(theta))
}

## K many simulated data from empirical RAPTT distribution under H0 based on R1
RAPTT.emp.h0.R1=function(n1,n2,p,Sigma,m,K)  
{ 

RAPTT.Stats=matrix(0,K,1)
for( j in 1:K)
{
mu1=rep(0, p)
mu2=rep(0, p)
X=rmvnorm(n1, mu1, Sigma)
Y=rmvnorm(n2, mu2, Sigma)
RAPTT.Stats[j]=RAPTT.Stats.R1(X,Y,m)
}
return(RAPTT.Stats)
}
## Declare the sample sizes and dimension to generate the observation from empirical null distn
n1=n2=50
p=200

##Generate K=1000 many empirical RAPTT stats under H0 to get the null distribution
E=RAPTT.emp.h0.R1(n1,n2,p,Sigma=diag(p),m=5000,K=1000)

RAPTT.R1.pvalue=function(X,Y,m,E) 
{ 
pvalue=mean(RAPTT.Stats.R1(X,Y,m=5000)>E)
return(pvalue)
}

#Choices of covariance matrices used in the simulation
Sigma1=diag(p)
Sigma2=diag(c(20/(1:20),rep(1,(p-20))))
Sigma3=toeplitz(c(1,0.4,rep(0,(p-2))))
library(matrixcalc)
b=0.85*diag(25)+0.15*rep(1,25)%*%t(rep(1,25))
Sigma4=b%s%b%s%b%s%b%s%b%s%b%s%b%s%b
#Specify Sigma
Sigma=Sigma1

##Choice of mean vector used in simulation for power study
mu1=rep(0, p)
mu2=rep(0, p)
a=0.01 ## Percentage chosen
I=sample(p,floor(p*a));
mu2[I]=rnorm(floor(p*a))+1;
#Alternative 1
mu2=mu2/sqrt(t(mu2)%*%solve(Sigma)%*%(mu2)/2)
#Alternative 2 
mu2=sqrt(sqrt(matrix.trace(Sigma%*%Sigma)))*sqrt(0.1)*(mu2/sqrt(t(mu2)%*%mu2));

#Generate data
X=rmvnorm(n1, mu1, Sigma)
Y=rmvnorm(n2, mu2, Sigma)

##p-value of RAPTT for any given data matrix X, Y

RAPTT.R1.pvalue(X,Y,m=5000,E)
